@@ -3,20 +3,359 @@
 from . import GeneralMap as GM
 from . import ScriptMap
 from aksharamukha.ScriptMap.Roman import Avestan
-from aksharamukha.ScriptMap.MainIndic import Tamil,Malayalam,Gurmukhi,Oriya,Saurashtra,Sinhala,Urdu,Devanagari, Chakma, Limbu
-from aksharamukha.ScriptMap.EastIndic import Tibetan, Thai, PhagsPa, ZanabazarSquare
+from aksharamukha.ScriptMap.MainIndic import Ahom, Tamil,Malayalam,Gurmukhi,Oriya,Saurashtra,Sinhala,Urdu,Devanagari, Chakma, Limbu, Takri, TamilExtended
+from aksharamukha.ScriptMap.EastIndic import Tibetan, Thai, PhagsPa, ZanabazarSquare, Burmese, KhamtiShan
 from . import ConvertFix as CF
 import re
+import functools
 
 ### Write Lotsssss of Comments
 ### Rewrite all ListC, ListV as sorted(List,key=len,reverse=True). Then Correctrulu may be unnecessary
 
 ### Consider Adding Options to ignore Nukta etc for Gujarati bengali by default
 
+##
+
 def default(Strng):
 
     return Strng
 
+def TamilStyleUUCore(Strng):
+    Strng = re.sub('([ഖഗഘഛഝഠഡഢഥദധഫബഭ])' + '([ുൂ])', r'\1' + '\u200D' + r'\2', Strng)
+
+    return Strng
+
+def TamilStyleUUOther(Strng):
+    Strng = re.sub('([ജശഷസഹ])' + '([ുൂ])', r'\1' + '\u200D' + r'\2', Strng)
+    Strng = re.sub('(ശ്ര)' + '([ുൂ])', r'\1' + '\u200D' + r'\2', Strng)
+    Strng = re.sub('(ശ്‍ര)' + '([ുൂ])', r'\1' + '\u200D' + r'\2', Strng)
+
+
+    return Strng
+
+def ContextualLLa(Strng):
+    ListVS = '|'.join(GM.CrunchSymbols(GM.VowelSigns, 'Tamil'))
+    ListC = '|'.join(GM.CrunchSymbols(GM.Consonants, 'Tamil'))
+
+    Strng = re.sub('(ஆவ)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = re.sub('(்ரவா)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = re.sub('(்ர)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = re.sub('(ாவ)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = re.sub('(யா)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = re.sub('([கத])' + '(' + ListVS + ')?' + '([³⁴])'+ 'ல', r'\1\2\3' +  'ள', Strng)
+    Strng = re.sub('(ஜு)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = re.sub('(தா|து)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = re.sub('(ரிம)'+ 'ல', r'\1' +  'ள', Strng)
+    Strng = Strng.replace('ள்ய', 'ல்ய')
+
+    Strng = re.sub('ந்' + '([\.।॥,!-])', 'ன்' + r'\1', Strng)
+    Strng = re.sub('ந்' + '(\s)', 'ன்' + r'\1', Strng)
+    Strng = re.sub('ந்$', 'ன்', Strng)
+
+    Strng = Strng.replace('ம்ʼக', 'ங்க')
+    Strng = Strng.replace('ம்ʼச', 'ஞ்ச')
+    Strng = Strng.replace('ம்ʼஜ', 'ஞ்ஜ')
+    Strng = Strng.replace('ம்ʼட', 'ண்ட')
+    Strng = Strng.replace('ம்ʼத', 'ந்த')
+
+    return Strng
+
+def AhomClosed(Strng):
+    vir = Ahom.ViramaMap[0]
+    anu = Ahom.AyogavahaMap[1]
+
+    #closed i
+    Strng = Strng.replace('\U00011722', '\U00011723')
+    Strng = re.sub('(\U00011723)(.)('+vir+')', '\U00011722'+r'\2\3', Strng)
+    Strng = Strng.replace(anu + '\U00011723', anu + '\U00011722')
+
+    #closed u
+    Strng = Strng.replace('\U00011724', '\U00011725')
+    Strng = re.sub('(\U00011725)(.)('+vir+')', '\U00011724'+r'\2\3', Strng)
+    Strng = Strng.replace(anu + '\U00011725', anu + '\U00011724')
+
+    #closed e
+    Strng = re.sub('(\U00011726\U00011727)(.)('+vir+')', '\U00011726'+r'\2\3', Strng)
+    Strng = Strng.replace('\U00011726\U0001172A\U00011727', anu + '\U00011727')
+
+    #closed o
+    Strng = re.sub('(\U00011726\U00011721)(.)('+vir+')', '\U00011728'+r'\2\3', Strng)
+    Strng = Strng.replace('\U00011726\U0001172A\U00011721', anu + '\U00011728')
+
+    return Strng
+
+def TeluguTamilZha(Strng):
+
+    return Strng
+
+def TeluguTamilRra(Strng):
+    Strng = Strng.replace('ఱ్ఱ', 'ౘ్ౘ')
+    Strng = Strng.replace('ఱ', 'ౘ')
+
+    return Strng
+
+def ThaiNativeConsonants(Strng):
+    Strng = Strng.replace('ท', 'ด')
+    Strng = Strng.replace('พ', 'บ')
+    Strng = Strng.replace("\u0E36","\u0E34\u0E4D")
+    Strng = Strng.replace('ํ', 'งฺ')
+
+    Strng = re.sub('(\u0E3A)([ยรลวห])', '\u035C'+ r'\2', Strng)
+    Strng = Strng.replace('ห\u0E3A', 'ห\u035C')
+
+    Strng = re.sub('([ยรลวห])' + '\u035C' + r'\1', r'\1' + '\u0E3A' + r'\1', Strng)
+
+    Strng = re.sub('(า)(.)(ฺ)', '็' + r'\1\2\3', Strng)
+    Strng = re.sub('([เโ])(.)(.)(ฺ)',  r'\1\2' + '็' +  r'\3\4', Strng)
+
+    Strng = ThaiTranscription(Strng, False)
+
+    Strng = Strng.replace('ะ͜', '\u035C')
+    Strng = Strng.replace('ะ็', '็')
+    Strng = re.sub('([เโไ])(.)(\u035C)(.)([ะ\u0E31])', r'\1\2\3\4', Strng)
+
+    Strng = Strng.replace('ค', 'ก\u0325')
+    Strng = Strng.replace('ช', 'จ\u0325')
+
+    Strng = Strng.replace('ง', 'งํ')
+
+    Strng = Strng.replace('ะงํ\u035C', '\u0E31งํ')
+
+    Strng = re.sub('([เโไ])(งํ)([าัะ])', r'\1' + 'ง' + r'\2', Strng)
+    Strng = re.sub('([เโไ])(งํ)', r'\1' + 'ง', Strng)
+    Strng = re.sub('(งํ)([าัะ])', 'ง' + r'\2', Strng)
+
+    return Strng
+
+def KhamiShanMyanmarNumerals(Strng):
+    for x, y in zip(KhamtiShan.NumeralMap, Burmese.NumeralMap):
+        Strng = Strng.replace(x, y)
+
+    return Strng
+
+def KhamtiShanRa(Strng):
+
+    Strng = Strng.replace('ရ', 'ꩳ')
+
+    return Strng
+
+def granthafinal(Strng):
+
+    return Strng
+
+def Dot2Dandas(Strng):
+    Strng = Strng.replace('..', '॥')
+    Strng = Strng.replace('.', '।')
+
+    return Strng
+
+def SaurastraHaaruColon(Strng):
+    vir = Tamil.ViramaMap[0]
+    ha = Tamil.ConsonantMap[-1]
+
+    Strng = Strng.replace(vir + ha, ':')
+
+    ListVS = '|'.join(GM.CrunchSymbols(GM.VowelSigns, 'Tamil'))
+
+    Strng = re.sub('(:)' + '(' + ListVS + ')', r'\2\1', Strng)
+
+    Strng = re.sub('(\s)(ன)', r'\1' + 'ந', Strng)
+    Strng = re.sub('^ன', 'ந', Strng)
+
+    return Strng
+
+def TamilExtendedNNA(Strng):
+
+    na = TamilExtended.ConsonantMap[19]
+    nna = TamilExtended.SouthConsonantMap[3]
+    vir = TamilExtended.ViramaMap[0]
+    ta = TamilExtended.ConsonantMap[15]
+
+    ListV = '|'.join(GM.CrunchSymbols(GM.Vowels+GM.VowelSigns+GM.Consonants,'TamilExtended')+[TamilExtended.SignMap[0]])
+
+    Strng = re.sub('('+ListV+')'+ GM.VedicSvaras + '('+na+')' + '(?!' + vir + ')',r'\1\2'+nna,Strng)
+    Strng = re.sub('('+ListV+')'+ GM.VedicSvaras + '('+na+')' + '(?!' + vir + ')',r'\1\2'+nna,Strng)
+
+    Strng = re.sub('(ന്‌)(?![തഥദധ])', 'ഩ്‌', Strng)
+
+    return Strng
+
+def TakriRemoveGemination(Strng):
+
+    Strng = re.sub('(.)' + Takri.ViramaMap[0] + r'\1', r'\1', Strng)
+
+    return Strng
+
+def MongolianSyllabize(Strng):
+    vowels = '(' + '|'.join(GM.CrunchSymbols(GM.Vowels, 'Mongolian')+['\u1820']) + ')'
+    consonants = '(' + '|'.join(GM.CrunchSymbols(GM.Consonants, 'Mongolian')) + ')'
+
+    Strng = re.sub(consonants + '?' + vowels, r'\1\2' + ' ', Strng)
+    Strng = re.sub('(\u180E\u1820)' + consonants, r'\1 \2', Strng)
+    Strng = re.sub('\u1820 ', '\u1820\u180B ', Strng)
+    Strng = Strng.replace('ᠣᠸᠠ᠋', 'ᠣᠸᠠ')
+    Strng = Strng.replace('ᠣᠸᠸᠠ᠋', 'ᠣᠸᠸᠠ')
+    Strng = Strng.replace(' \u180E', '\u180E')
+    Strng = Strng.replace(' ' + '\u200B', '')
+    Strng = Strng.replace(' ᢁ', 'ᢁ')
+
+    return Strng
+
+def TibetanSyllabize(Strng):
+    vowels = '(' + '|'.join(GM.CrunchSymbols(GM.Vowels, 'Tibetan')) + ')'
+    consonants = '(' + '|'.join(GM.CrunchSymbols(GM.Consonants, 'Tibetan')+['ཨ','ཅ','ཆ','ཇ','ཇྷ']) + ')'
+    vowelsigns = '(' + '|'.join(GM.CrunchSymbols(GM.VowelSigns, 'Tibetan')+['\u0F80']) + ')'
+    combiningSigns = '(' + '|'.join(GM.CrunchSymbols(GM.CombiningSigns, 'Tibetan')+['\u0F82']) + ')'
+    ListSubC = '(' + '|'.join([chr(x+80) for x in range(0x0F40,0x0F68)] + ['ྻ','ྺ','ྼ']) + ')' # Subjoined Consonants
+
+    Strng = re.sub(vowelsigns + combiningSigns + '?', r'\1\2་', Strng)
+    Strng = re.sub(consonants , r'\1་', Strng)
+    Strng = re.sub(ListSubC, r'\1་', Strng)
+    Strng = re.sub('་' + vowelsigns, r'\1', Strng)
+    Strng = re.sub('་' + ListSubC, r'\1', Strng)
+    Strng = re.sub('་' + combiningSigns, r'\1', Strng)
+    Strng = re.sub(combiningSigns, r'\1་', Strng)
+
+    Strng = Strng.replace('་་', '་')
+
+    return Strng
+
+def SoyomboSyllabize(Strng):
+    vowels = '(' + '|'.join(GM.CrunchSymbols(GM.Vowels, 'Soyombo')) + ')'
+    consonants = '(' + '|'.join(GM.CrunchSymbols(GM.Consonants, 'Soyombo')+['𑩐', '\U00011A83']) + ')'
+    vowelsigns = '(' + '|'.join(GM.CrunchSymbols(GM.VowelSigns, 'Soyombo')) + ')'
+    combiningSigns = '(' + '|'.join(GM.CrunchSymbols(GM.CombiningSigns, 'Soyombo')) + ')'
+
+    fin = '(' + '|'.join(['\U00011A8A','\U00011A8B','\U00011A8C','\U00011A8D','\U00011A8E','\U00011A8F','\U00011A90','\U00011A91','\U00011A92','\U00011A93','\U00011A94']) + ')'
+
+    Strng = re.sub(vowelsigns + combiningSigns + '?', r'\1\2 ', Strng)
+    Strng = re.sub(consonants , r'\1 ', Strng)
+    Strng = re.sub(' ' + vowelsigns, r'\1', Strng)
+    Strng = re.sub(' ' + combiningSigns, r'\1', Strng)
+    Strng = re.sub('\U00011A99' + ' ', '\U00011A99', Strng)
+    Strng = re.sub(combiningSigns, r'\1 ', Strng)
+    Strng = re.sub(' 𑪘', '\U00011A98', Strng)
+    Strng = re.sub(fin, r'\1 ', Strng)
+    Strng = re.sub('( )' + fin, r'\2 ', Strng)
+    #Strng = re.sub(combiningSigns, r'\1་', Strng)
+
+    return Strng
+
+
+def TakriArchaicKha(Strng):
+
+    return Strng.replace('𑚸', '𑚋')
+
+def TeluguReph(Strng):
+    consonants = '(' + '|'.join(GM.CrunchSymbols(GM.Consonants, 'Telugu')) + ')'
+    Strng = re.sub('ర్' + consonants, 'ర్‍' + r'\1', Strng)
+    Strng = Strng.replace('\u0C4Dర్‍', '\u0C4Dర్')
+
+    return Strng
+
+def PhagsPaTib(Strng):
+
+    return Strng
+
+def PhagsPaSeal(Strng):
+
+    return Strng
+
+def TamilExtendedAnusvara(Strng):
+    Strng = AnusvaraToNasal(Strng, 'TamilExtended')
+    Strng = Strng.replace('\u0D02', 'മ്‌')
+
+    return Strng
+
+def RomanReadableLongEO(Strng):
+
+    Strng = Strng.replace('o', 'oa')
+    Strng = Strng.replace('oa\'', 'o')
+
+    Strng = Strng.replace('e', 'ae')
+    Strng = Strng.replace('ae\'', 'e')
+
+    Strng = Strng.replace('aeae', 'ee')
+    Strng = Strng.replace('oaoa', 'oo')
+
+    return Strng
+
+def TeluguArasunnaChandrabindu(Strng):
+    Strng = Strng.replace('ఀ', 'ఁ')
+
+    return Strng
+
+def MarchenSanskritPalatals(Strng):
+    tsaSeries = ['\U00011C82', '\U00011C83', '\U00011C84']
+    jaSereis =  ['\U00011C76', '\U00011C77', '\U00011C78']
+
+    for x, y in zip(tsaSeries, jaSereis):
+        Strng = Strng.replace(x, y)
+
+    return Strng
+
+
+def SoyomboSanskritPalatals(Strng):
+    tsaSeries = ['𑩵','𑩶','𑩷']
+    caSeries = ['𑩡','𑩢','𑩣']
+
+    for x, y in zip(tsaSeries,caSeries):
+        Strng = Strng.replace(x, y)
+
+    return Strng
+
+def TibetanSanskritPalatals(Strng):
+    caSeries = ['ཅ','ཆ','ཇ','ཇྷ']
+    tsaSeries = ['ཙ','ཚ','ཛ','ཛྷ']
+
+    for x, y in zip(tsaSeries,caSeries):
+        Strng = Strng.replace(x, y)
+
+    return Strng
+
+def ZanabazarSanskritPalatals(Strng):
+    tsaSeries = ['𑨣', '𑨤', '𑨥']
+    caSeries = ['𑨐','𑨑','𑨒']
+
+    for x, y in zip(tsaSeries,caSeries):
+        Strng = Strng.replace(x, y)
+
+    return Strng
+
+def SoyomboFinals(Strng):
+
+    return Strng
+
+def SoyomboInitials(Strng):
+    viraCon = ['\U00011A7C\U00011A99', '\U00011A7D\U00011A99', '\U00011A81\U00011A99']
+    initial = ['\U00011A86', '\U00011A87', '\U00011A89']
+
+    for x, y in zip(viraCon, initial):
+        Strng = Strng.replace(x, y)
+
+    return Strng
+
+def ZanzabarSpaceTsheg(Strng):
+    Strng = Strng.replace(' ', '\U00011A41')
+
+    return Strng
+
+def SoyomboSpaceTscheg(Strng):
+    Strng = Strng.replace(' ', '\U00011A9A')
+
+    return Strng
+
+def NasaltoAnsvaraIASTISO(Strng):
+    Strng = Strng.replace('ṁ', 'ṃ')
+
+    Strng = re.sub('(ṃ)(k|g)', 'ṅ' + r'\2', Strng)
+    Strng = re.sub('(ṃ)(c|j)', 'ñ' + r'\2', Strng)
+    Strng = re.sub('(ṃ)(ṭ|ḍ)', 'ṇ' + r'\2', Strng)
+    Strng = re.sub('(ṃ)(t|d)', 'n' + r'\2', Strng)
+    Strng = re.sub('(ṃ)(p|b)', 'm' + r'\2', Strng)
+
+    return Strng
+
 def removeDiacritics(Strng):
     diacritics = ['\u0331', '\u0306', '\u0323', '\u035F', '\u0324', '\u035F', '\u0307', '\u0301', '\u0303', '\u0310', '\u0306', '\u0302', '\u0304']
 
@@ -39,6 +378,9 @@ def ranjanawartu(Strng):
     Strng = Strng.replace('་', '࿎ ')
     return Strng
 
+def TaiKuen(Strng):
+    return Strng
+
 def TaiThamLao(Strng):
     return Strng
 
@@ -54,6 +396,72 @@ def oldtamilortho(Strng):
 def nepaldevafont(Strng):
     return Strng
 
+def granthaserif(Strng):
+    return Strng
+
+def ChakmaPali(Strng):
+    listC = '('+"|".join(sorted(GM.CrunchSymbols(GM.Consonants,"Chakma")+Chakma.VowelMap[:1],key=len,reverse=True))+')'
+    listV = '('+"|".join(sorted(GM.CrunchSymbols(GM.VowelSigns,"Chakma")+Chakma.ViramaMap+['\U00011133'],key=len,reverse=True))+')'
+
+    Strng = ChakmaGemination(Strng, reverse = True)
+
+    Strng = Strng.replace('𑄤', '\U00011147') # Replace Ya
+    Strng = Strng.replace('𑄡', '𑄠') # Replace vA
+
+    ## reverse A introduction
+
+    Strng = Strng.replace("\U00011127","\u02BE")
+    Strng = re.sub("("+listC+")"+"(?!"+listV+'|\u02BE'+")",r'\1''\U00011127',Strng)
+    Strng = Strng.replace("\u02BE","")
+
+    ## Replace A with Visarga as per Pali
+
+    Strng = Strng.replace('\U00011127', '\U00011102')
+
+    ## Replace subjoining with Explicit Virama
+
+    Strng = Strng.replace('\U00011133', '\U00011134')
+
+    return Strng
+
+def ThaiSajjhayawithA(Strng):
+    Strng = ThaiSajjhayaOrthography(Strng)
+    Strng = Strng.replace('ัง','ังฺ')
+    Strng = ThaiTranscription(Strng, anusvaraChange = True)
+
+    Strng = Strng.replace('ะํ', 'ํ')
+    Strng = Strng.replace('ะั', 'ั')
+    Strng = Strng.replace('ะ๎', '๎')
+
+    Strng = re.sub('([เโไ])(.๎)([ยรลวศษสหฬ])ะ', r'\1\2\3', Strng)
+
+    Strng = Strng.replace("\u0E32\u0E4D", "\u0E33").replace("\u0E34\u0E4D", "\u0E36") # reverse AM, iM
+
+    return Strng
+
+def LaoSajjhaya(Strng):
+    Strng = ThaiSajjhayaOrthography(Strng, Script = "LaoPali")
+
+    return Strng
+
+def LaoSajjhayawithA(Strng):
+    Strng = ThaiSajjhayaOrthography(Strng, Script = "LaoPali")
+    Strng = Strng.replace('ັງ', 'ັງ຺')
+    Strng = CF.LaoPaliTranscribe(Strng, anusvaraChange = True)
+
+    Strng = Strng.replace('ະໍ', 'ໍ')
+    Strng = Strng.replace('ະັ', 'ັ')
+    Strng = Strng.replace('ະ๎', '๎')
+
+    Strng = Strng.replace('ະ໌', '໌')
+    Strng = Strng.replace('ະົ', 'ົ')
+
+    Strng = re.sub('([ເໂໄ])(.๎)([ຍຣລວຨຩສຫຬ])ະ', r'\1\2\3', Strng)
+
+    Strng = Strng.replace('າໍ', 'ຳ')
+
+    return Strng
+
 def UseAlternateVSU(Strng):
     Strng = Strng.replace('𑖲', '𑗜')
 
@@ -112,7 +520,7 @@ def ChakmaEnableAllConjuncts(Strng):
 
 def ChakmaGemination(Strng, reverse = False):
     ListC = "(" + "|".join(GM.CrunchSymbols(GM.Consonants, 'Chakma')) + ")"
-    virs = "([\U00011134 \U00011133])"
+    virs = "([\U00011134\U00011133])"
     virExp = "\U00011134"
     virDep = "\U00011133"
     ListV = '('+"|".join(sorted(GM.CrunchSymbols(GM.VowelSignsNV,"Chakma"), key=len, reverse = True)) + ")"
@@ -193,6 +601,7 @@ def LimbuSpellingSaI(Strng):
 
     for x, y in zip(FCons, FinalCons):
         Strng = Strng.replace('\u193A' + y, x)
+        Strng = Strng.replace('\u193A\u1922' + y, '\u1922' + x)
 
     return Strng
 
@@ -274,10 +683,10 @@ def DevanagariPrishtamatra(Strng, reverse = False):
         Strng = Strng.replace('े','ॎ')
         Strng = Strng.replace('ै','ॎे')
         Strng = Strng.replace('ो','ॎा')
-        Strng = Strng.replace('ौ','ॎाे')
+        Strng = Strng.replace('ौ','ॎो')
     else:
         Strng = Strng.replace('ॎे', 'ै')
-        Strng = Strng.replace('ॎाे', 'ौ')
+        Strng = Strng.replace('ॎो', 'ौ')
         Strng = Strng.replace('ॎा', 'ो')
         Strng = Strng.replace('ॎ', 'े')
 
@@ -342,6 +751,13 @@ def GranthaPrakrit(Strng):
     Strng = Strng.replace("𑌂", "𑌀")
     Strng = InsertGeminationSign(Strng, 'Grantha')
 
+    ## not at the beginning of words
+    pat = r'\s𑌂.'
+    Strng = functools.reduce(lambda s, m: s.replace(m, ReverseGeminationSign(m, 'Grantha')), re.findall(pat, Strng), Strng)
+
+    pat = r'𑍍𑌂.'
+    Strng = functools.reduce(lambda s, m: s.replace(m, ReverseGeminationSign(m, 'Grantha')), re.findall(pat, Strng), Strng)
+
     return Strng
     ## Insert Gemination Sign
 
@@ -368,15 +784,24 @@ def TamilNaToNNa(Strng):
 
     ListV = '|'.join(GM.CrunchSymbols(GM.Vowels+GM.VowelSigns+GM.Consonants,'Tamil')+[Tamil.SignMap[0].replace('(','\(').replace(')','\)')])
 
-    Strng = re.sub('('+ListV+')'+'('+na+')',r'\1'+nna,Strng)
+    Strng = re.sub('('+ListV+')'+ GM.VedicSvaras + '('+na+')' + '(?!' + vir + ')',r'\1\2'+nna,Strng)
+    Strng = re.sub('('+ListV+')'+ GM.VedicSvaras + '('+na+')' + '(?!' + vir + ')',r'\1\2'+nna,Strng)
+
+    #Strng = re.sub('(ந்)(?!த)', 'ன்', Strng)
+
     Strng = re.sub('(²|³|⁴)'+'('+na+')',r'\1'+nna,Strng)
     Strng = Strng.replace(nna+vir+ta,na+vir+ta)
 
+    Strng = re.sub('(\s)(ன)', r'\1' + 'ந', Strng)
+    Strng = re.sub('^ன', 'ந', Strng)
+
+    Strng = re.sub("(?<=ஶ்ரீ)(ன)(?!" + vir + ")", "ந", Strng)
+
     return Strng
 
 # കൽന് കത്ല് ക്ഷേത്ര് കൻല് - Check this
 
-def MalayalamChillu(Strng, reverse=False):
+def MalayalamChillu(Strng, reverse=False, preserve=False):
 
     Chillus=['\u0D7A','\u0D7B','\u0D7C','\u0D7D','\u0D7E', 'ഩ‍്']
 
@@ -405,10 +830,16 @@ def MalayalamChillu(Strng, reverse=False):
     if not reverse:
         for i in range(len(Chillus)):
             #print '(?<!'+'['+vir+''.join(Chillus)+']'+')'+'('+ConVir[i]+')'+'(?!['+''.join(CList[i])+'])'
-            Strng = re.sub(ListC + '('+ConVir[i]+')'+'(?!['+''.join(CList[i])+'])',r'\1' + Chillus[i],Strng)
+            Strng = re.sub(ListC + GM.VedicSvaras + '('+ConVir[i]+')'+'(?!['+''.join(CList[i])+'])',r'\1\2' + Chillus[i],Strng)
     else:
-        for x,y in zip(Chillus, ConVir):
-            Strng = Strng.replace(x, y +'ˍ') ## Fix the reversal of characters of this
+        print('called mal')
+        print(preserve)
+        if preserve:
+            for x,y in zip(Chillus, ConVir):
+                Strng = Strng.replace(x, y +'ˍ') ## Fix the reversal of characters of this
+        else:
+            for x,y in zip(Chillus, ConVir):
+                Strng = Strng.replace(x, y) ## Fix the reversal of characters of this
 
     return Strng
 
@@ -520,8 +951,11 @@ def NasalToAnusvara(Strng,Target):
 
     for i in range(len(ListN)):
         #print '('+ListN[i]+')'+'('+vir+')'+'('+ListC[i]+')'
-        Strng = re.sub(ListCAll + '(?<!' + vir + ')' + '('+ListN[i]+')'+'('+vir+')'+'('+ListC[i]+')',r'\1'+Anu+r'\4',Strng)
-        Strng = re.sub(ListCAll + '(?<!' + vir + ')' + '('+ListN[i]+')'+'('+vir+')'+'('+ListC[i]+')',r'\1'+Anu+r'\4',Strng)
+        Strng = re.sub(ListCAll + GM.VedicSvaras + '(?<!' + vir + ')' + '('+ListN[i]+')' +'('+vir+')'+'('+ListC[i]+')',r'\1\2'+Anu+r'\5',Strng)
+        Strng = re.sub(ListCAll + GM.VedicSvaras + '(?<!' + vir + ')' + '('+ListN[i]+')' +'('+vir+')'+'('+ListC[i]+')',r'\1\2'+Anu+r'\5',Strng)
+
+    for svara in GM.VedicSvarasList:
+        Strng = Strng.replace(svara + Anu, Anu + svara)
 
     return Strng
 
@@ -539,7 +973,10 @@ def AnusvaraToNasal(Strng,Target):
     Anu = GM.CrunchSymbols(GM.CombiningSigns,Target)[1]
 
     for i in range(len(ListN)):
-        Strng = re.sub('('+Anu+')'+'('+ListC[i]+')',ListN[i]+vir+r'\2',Strng)
+        Strng = re.sub('('+Anu+')'+ GM.VedicSvaras + '('+ListC[i]+')',ListN[i]+vir+r'\2\3',Strng)
+
+        if Target == "Tamil":
+            Strng = re.sub('(ம்)'+ GM.VedicSvaras + '(ʼ)' + '('+ListC[i]+')',ListN[i]+vir+r'\2\4',Strng)
 
     return Strng
 
@@ -585,7 +1022,10 @@ def MToAnusvara(Strng,Target):
 
     ListCAll = '(' + '|'.join(GM.CrunchSymbols(GM.Characters, Target)) + ')'
 
-    Strng = re.sub(ListCAll + '(?<!' + vir + ')'+'('+M+')'+'(?!'+ListC+')',r'\1'+Anusvara,Strng)
+    Strng = re.sub(ListCAll + GM.VedicSvaras + '(?<!' + vir + ')'+'('+M+')'+'(?!'+ListC+')',r'\1\2'+Anusvara,Strng)
+
+    for svara in GM.VedicSvarasList:
+        Strng = Strng.replace(svara + Anusvara, Anusvara + svara)
 
     #Strng = Strng.replace(M,Anusvara)
 
@@ -618,7 +1058,7 @@ def YaToYYa(Strng,Target):
     ListVarga = '|'.join(GM.CrunchList('ConsonantMap',Target)[0:25])
 
     if Target in ['Assamese','Bengali', 'Oriya', "Chakma"]:
-        Strng = re.sub('('+ListC+')'+Ya,r'\1'+YYa,Strng)
+        Strng = re.sub('('+ListC+')'+ GM.VedicSvaras + Ya,r'\1\2'+YYa,Strng)
 
         if Target in ['Assamese', 'Bengali']:
             Strng = Strng.replace(vir+YYa,vir+Ya)
@@ -889,7 +1329,7 @@ def TibetanRemoveBa(Strng):
 
     return Strng
 
-def ThaiLaoTranscription(Strng,Script,shortA,shortAconj,reverse=False):
+def ThaiLaoTranscription(Strng,Script,shortA,shortAconj,reverse=False, anusvaraChange=True):
     ## For Native lao: aMDa give an'da as intermediate (N doesn't exist in Native Lao )
     ## Hence issues with nasal conversion
 
@@ -897,6 +1337,9 @@ def ThaiLaoTranscription(Strng,Script,shortA,shortAconj,reverse=False):
 
     cons = "|".join(GM.CrunchSymbols(GM.Consonants, Script)+GM.CrunchList('VowelMap',Script)[0:1])
 
+    if Script == 'Thai':
+        cons = "|".join(GM.CrunchSymbols(GM.Consonants, Script)+GM.CrunchList('VowelMap',Script)[0:1] + ['ฮ', 'บ', 'ฝ', 'ด'])
+
     if Script == 'Lao':
         cons = "|".join(GM.CrunchSymbols(GM.Consonants, Script) + GM.CrunchList('VowelMap',Script)[0:1] + ['ດ','ບ','ຟ'])
 
@@ -910,18 +1353,24 @@ def ThaiLaoTranscription(Strng,Script,shortA,shortAconj,reverse=False):
 
     vowA = GM.CrunchList('VowelMap',Script)[0]
 
-    Strng = AnusvaraToNasal(Strng,Script)
+    if anusvaraChange:
+        Strng = AnusvaraToNasal(Strng,Script)
 
     if not reverse:
-        if Script == Thai:
+        if Script == 'Thai':
             Strng = re.sub("(["+EAIO+"])"+"("+cons+")"+"("+vir+")",r'\2\3\1',Strng) #Reverse bre, bro etc
             Strng = Strng.replace("\u0E33","\u0E32\u0E4D").replace("\u0E36","\u0E34\u0E4D") # reverse AM, iM
+        if Script == 'LaoPali':
+            Strng = Strng.replace('ຳ', 'າໍ')
+
+        if anusvaraChange:
+            Strng = Strng.replace(Anu, ng + vir)
 
         Strng = re.sub("(?<!["+EAIO+"])"+"("+cons+")"+"(?!["+AIUVir+"])",r'\1'+shortA,Strng)
-        Strng = Strng.replace(Anu,ng)
-        Strng = Strng.replace(vir,"")
-        Strng = re.sub("("+shortA+")"+"(?=("+cons+")"+"("+cons+"|"+"["+EAIO+"]))",shortAconj,Strng)
-        Strng = re.sub("("+shortA+")"+"(?="+ng+")(?!["+AIUVir+"])",shortAconj,Strng)
+        Strng = re.sub("("+shortA+")"+"(?=("+cons+")"+"("+vir+"))",shortAconj,Strng)
+        Strng = Strng.replace(vir, '')
+
+        ## Fix Purevowels
 
     else:
         consOnly = "|".join(GM.CrunchSymbols(GM.Consonants, Script))
@@ -955,16 +1404,67 @@ def ThaiVisargaSaraA(Strng):
 
     return Strng
 
-def ThaiTranscription(Strng):
+def ThaiSajjhayaOrthography(Strng, Script = "Thai"):
+    ## reverse digraphs
+    Strng = CF.ThaiReverseVowelSigns(Strng, True)
+    Strng = CF.ThaiDigraphConjuncts(Strng, True)
+    Strng = CF.ThaiReverseVowelSigns(Strng)
+
+    if Script == "Thai":
+        Strng = Strng.replace('ฺ', '์')
+    if Script == "LaoPali":
+        Strng = Strng.replace('຺', '์')
+
+    cons = "|".join(GM.CrunchSymbols(GM.Consonants, Script)+GM.CrunchList('VowelMap',Script)[0:1])
+    EAIO = "".join(GM.CrunchList('VowelSignMap',Script)[9:12]+GM.CrunchList('SinhalaVowelSignMap',Script)[:])
+
+    # short a for conjuncts : t(a)ssa
+    Strng = re.sub('(?<![' + EAIO + '])' + '(' + cons + ')' + '(' + cons + ')' + '(์)', r'\1' + 'ั' + r'\2\3', Strng)
+
+    if Script == "Thai":
+        cons_others  = '([ยรลวศษสหฬ])' # avarga
+    if Script == "LaoPali":
+        cons_others = '([ຍຣລວຨຩສຫຬ])' # avarga
+
+    Strng = re.sub('(?<![' + EAIO + '])' + '(' + cons + ')' + '(' + cons + ')' + '(์)', r'\1' + 'ั' + r'\2\3', Strng)
+
+    # varga + avaraga or avarga + varga add joiner
+    # hma, mha etc.
+    Strng = re.sub('(' + cons + ')' + '(์)' + '([' + EAIO + ']?)' + cons_others , r'\1' + '๎' + r'\3\4', Strng)
+    Strng = re.sub(cons_others + '(์)' + '([' + EAIO + ']?)' + '(' + cons + ')', r'\1' + '๎' + r'\3\4', Strng)
+
+    ## ssa, lla, nna do no add joiner
+    Strng = re.sub(cons_others + '(๎)' + '([' + EAIO + ']?)' + r'\1' , r'\1' + '์' + r'\3\1', Strng)
+
+    #reorder dve sme
+    Strng = re.sub('(' + cons  + ')' + '(๎)' + '([' + EAIO + '])' + '(' + cons + ')', r'\3\1\2\4', Strng)
+
+    if Script == "Thai":
+        Strng = Strng.replace('ง์', 'ง')
+
+    if Script == "LaoPali":
+        Strng = Strng.replace('ั', 'ັ')
+        Strng = Strng.replace("ງ์", "ງ")
+        Strng = Strng.replace("์", "໌")
+
+    #Strng = re.sub('([ยรลวศษสหฬ])(์)', r'\1' + '๎', Strng)
+
+    return Strng
+
+
+def ThaiTranscription(Strng, anusvaraChange = True):
 
     ## reverse digraphs
     Strng = CF.ThaiReverseVowelSigns(Strng, True)
     Strng = CF.ThaiDigraphConjuncts(Strng, True)
     Strng = CF.ThaiReverseVowelSigns(Strng)
 
-    Strng = ThaiLaoTranscription(Strng,"Thai", '\u0E30', '\u0E31')
+    Strng = ThaiLaoTranscription(Strng,"Thai", '\u0E30', '\u0E31', anusvaraChange = anusvaraChange)
 
     Strng = Strng.replace('ะ์','์')
+
+    Strng = Strng.replace('ะงัง', '\u0E31งํ')
+
 #    shortA = u'\u0E30'
 #    shortAconj = u'\u0E31'
 #    cons = "|".join(GM.CrunchSymbols(GM.Consonants, "Thai")+Thai.VowelMap[0:1])
@@ -1165,7 +1665,7 @@ def NewaSpecialTa(Strng):
     return Strng
 
 def TamilDisableSHA(Strng):
-    Strng = Strng.replace('ஶ', 'ஸ²')
+    Strng = Strng.replace('ஶ', 'ஷ²')
     Strng = CF.ShiftDiacritics(Strng,'Tamil')
 
     return Strng
